//Source file: C:\\sw_dev\\chat\\ClientDeChat_v1.java

import java.net.*;
import java.io.*;
import java.util.*;
import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import java.util.Properties;

/**
 * Client de chat simplu - aplicatie de sine statatoare
 * Aplicatie grafica Swing (extinde JFrame) 
 * care poate lansa in executie un fir nou (implementeaza Runnable)
 */
public class ClientDeChat_v1plus extends JFrame implements Runnable {
  private String adresaLocala;
  private int portLocal;
  private String adresaServer = null;
  private int portServer = 0;
  private boolean neconectat = true;
  
  /**  Flux de intrare dinspre retea */
  private DataInputStream inRetea;
  
  /**
   * Zona de text (configurata ca non-editabila)
   */
  private JTextArea outTextGrafic;
  
  /**
   * Intrare de text (editabila)
   */
  private JTextField inTextGrafic;

  private JButton stabilireAdresaServer;
  private JTextField intrareAdresaServer;

  private JButton stabilirePortServer;
  private JTextField intrarePortServer;

  private JButton stabilireNumeUser;
  private JTextField intrareNumeUser;

  // Obtinerea valorii variabilei de mediu "USER"
  // de la obiectului care incapsuleaza proprietatile sistemului de 
  // operare (obtinerea variabilelor mediului de executie)
  private String numeUser = System.getProperties().getProperty("user.name");


  /**
   * Fir de executie
   */
  private Thread firReceptie;
  
  /**
   * Socket flux (TCP)
   */
  private Socket socket;
  
  /**
   * Flux de iesire catre retea
   */
  private DataOutputStream outRetea;
  private InetAddress localHost;
  
  /**
   * Initializeaza obiectul de tip ClientDeChat_v1
   * @param title      Titlul ferestrei
   * @param inRetea    Flux de intrare dinspre retea
   * @param outRetea   Flux de iesire catre retea
   */
  public ClientDeChat_v1plus() {
    // Stabilire titlu fereastra (JFrame)
    super ();                                   

    // Containerul ferestrei grafice
    Container containerFereastra = this.getContentPane();

    // Stabilirea modului de asezare a componentelor in fereastra (layoutului) 
    containerFereastra.setLayout(new BorderLayout());

    // Zona de text non-editabila de iesire (cu posibilitati de defilare)
    outTextGrafic = new JTextArea(8, 40);
    outTextGrafic.setEditable (false);
    JScrollPane panouDefilabil = new JScrollPane(outTextGrafic,
                                 JScrollPane.VERTICAL_SCROLLBAR_ALWAYS, 
                                 JScrollPane.HORIZONTAL_SCROLLBAR_ALWAYS);
    containerFereastra.add("Center", panouDefilabil);

    // Camp de text editabil de intrare (initial non-editabil)
    inTextGrafic = new JTextField(40);
    containerFereastra.add("South", inTextGrafic);
    inTextGrafic.setEditable(false);

    // Container nou pentru zona superioara a ferestrei grafice
    JPanel panouSuperior = new JPanel();
    containerFereastra.add("North", panouSuperior);

    // Stabilirea modului de asezare a componentelor in zona superioara
    panouSuperior.setLayout(new GridLayout(3,2));

    // Camp de text pentru introducerea adresei serverului
    intrareAdresaServer = new JTextField(20);
    panouSuperior.add(intrareAdresaServer);

    // Buton pentru confirmarea introducerii adresei serverului
    stabilireAdresaServer = new JButton("Adresa server");
    panouSuperior.add(stabilireAdresaServer);

    // Camp de text pentru introducerea portului serverului
    intrarePortServer = new JTextField(20);
    panouSuperior.add(intrarePortServer);

    // Buton pentru confirmarea introducerii portului serverului
    stabilirePortServer = new JButton("Port server");
    panouSuperior.add(stabilirePortServer);

    // Camp de text pentru introducerea numelui utilizatorului
    intrareNumeUser = new JTextField(20);
    panouSuperior.add(intrareNumeUser);

    // Buton pentru confirmarea introducerii numelui utilizatorului
    stabilireNumeUser = new JButton("Nume utilizator");
    panouSuperior.add(stabilireNumeUser);

    System.out.println("\n Configurata interfata grafica \n");      

    //
    // Inregistrarea "ascultatorilor" de "evenimente actionare"  
    //
    inTextGrafic.addActionListener(ascultatorEvenimenteActionare);

    stabilireAdresaServer.addActionListener(ascultatorEvenimenteActionare);
    stabilirePortServer.addActionListener(ascultatorEvenimenteActionare);
    stabilireNumeUser.addActionListener(ascultatorEvenimenteActionare);

    intrareAdresaServer.addActionListener(ascultatorEvenimenteActionare);
    intrarePortServer.addActionListener(ascultatorEvenimenteActionare);
    intrareNumeUser.addActionListener(ascultatorEvenimenteActionare);



    // Inregistrarea "ascultatorului" de "evenimente fereastra" la "sursa" 
    // (fereastra curenta)
    this.addWindowListener(ascultatorInchidere);

    // Impachetarea (compactarea) componentelor in container
    pack();                          

    // Fereastra devine vizibila - echivalent cu frame.setVisible(true)
    show();                     
    // Cerere focus pe intrarea de text din fereastra curenta     
    inTextGrafic.requestFocus();

    // Asteptarea conectarii (pentru a lansa firul de executie de receptie)
    while (neconectat) {  }

    // Fir de executie pentru receptia mesajelor de la server
    firReceptie = new Thread (this); 
    // Lansarea firului de executie - se executa run()   
    firReceptie.start ();   
  }

  /**
    * Crearea unui "adaptor pentru ascultator" de "evenimente fereastra" 
    * Obiect al unei clase "anonime" care extinde clasa WindowAdapter
    */
  WindowAdapter ascultatorInchidere = new WindowAdapter() {
    /**
      * Tratarea inchiderii ferestrei curente
      */
    public void windowClosing(WindowEvent ev) {
      // Daca mai exista firul de executie de receptie
      if (firReceptie != null) {
        // Inchiderea firului de receptie 
        firReceptie.stop ();                   
      }
      // Terminarea programului
      System.exit(0);                                    
    }
  };

  /**
    * Crearea unui "ascultator" de "evenimente actionare"
    * Obiect al unei clase "anonime" care implementeaza interfata ActionListener
    */
  ActionListener ascultatorEvenimenteActionare = new ActionListener() {

    // Tratarea actionarii ascultatorilor inregistrati
    public void actionPerformed(ActionEvent ev) {

      // Actionarea intrarii de text (apasarea tastei "Enter")
      if (ev.getSource() == inTextGrafic) {

        // Citire mesajului din intrarea de text
        String intrare = inTextGrafic.getText();                 

        try {                      
          // Scrierea mesajului pe fluxul de iesire spre retea                                    
          outRetea.writeUTF(numeUser + " [" + adresaLocala + ":" + 
                    portLocal + "]> " + intrare); 
          // Fortarea trimiterii mesajului (fortarea golirii bufferului)
          outRetea.flush ();           
        } 
        // In cazul unei erori legata de conexiune
        catch (IOException ex) {
          // Afisarea exceptiei
          ex.printStackTrace(); 
          // Inchiderea firului de executie care efectueaza receptia
          firReceptie.stop ();     
        }
        // Pregatirea intrarii de text pentru noul mesaj (golirea intrarii)
        inTextGrafic.setText (""); 
      }

      // 
      else if ((ev.getSource() == stabilirePortServer) || 
               (ev.getSource() == intrarePortServer)) {
        try {                      
          portServer = Integer.parseInt(intrarePortServer.getText());
          intrarePortServer.setEditable(false);
      
          if (adresaServer != null) {

            // Crearea socketului catre server 
            socket = new Socket (adresaServer, portServer);

            System.out.println ("Client TCP lansat catre server [" + 
                socket.getInetAddress() + ":" + socket.getPort() + "]...");
            System.out.println ("pe portul local: " + socket.getLocalPort());

            inRetea = new DataInputStream (new 
                        BufferedInputStream (socket.getInputStream()));
            outRetea = new DataOutputStream (new 
                         BufferedOutputStream (socket.getOutputStream()));
            neconectat = false;
            inTextGrafic.setEditable(true);
            outTextGrafic.setText(""); 
          }
        }
        // In cazul unei erori legata de conexiune
        catch (IOException ex) {
          // Afisarea exceptiei
          ex.printStackTrace(); 
          intrarePortServer.setEditable(true);
          intrarePortServer.setText("");
          intrareAdresaServer.setEditable(true);
          intrareAdresaServer.setText("");
          outTextGrafic.append("Conectarea a esuat. " + 
              "Incercati alta adresa si alt numar de port.\n");
          portServer = 0;
          adresaServer = null;
        }
        catch (NumberFormatException ex) {
          // Afisarea exceptiei
          ex.printStackTrace(); 
          intrarePortServer.setText("");
          outTextGrafic.append("Formatul numarului de port trebuie sa fie " +
                   " numeric\n");
          portServer = 0;
        }
      } 

      // 
      else if ((ev.getSource() == stabilireAdresaServer) || 
               (ev.getSource() == intrareAdresaServer)) {
        adresaServer = intrareAdresaServer.getText();
        intrareAdresaServer.setEditable(false);
      
        if (portServer != 0) {
          try {                      
            // Crearea socketului catre server 
            socket = new Socket (adresaServer, portServer);

            System.out.println ("Client TCP lansat catre server [" + 
                socket.getInetAddress() + ":" + socket.getPort() + "]...");
            System.out.println ("pe portul local: " + socket.getLocalPort());

            inRetea = new DataInputStream (new 
                        BufferedInputStream (socket.getInputStream()));
            outRetea = new DataOutputStream (new 
                         BufferedOutputStream (socket.getOutputStream()));
            neconectat = false;
            inTextGrafic.setEditable(true);
            outTextGrafic.setText(""); 
          }
          // In cazul unei erori legata de conexiune
          catch (IOException ex) {
            // Afisarea exceptiei
            ex.printStackTrace(); 
            intrarePortServer.setEditable(true);
            intrarePortServer.setText("");
            intrareAdresaServer.setEditable(true);
            intrareAdresaServer.setText("");
            outTextGrafic.append("Conectarea a esuat. " + 
              "Incercati alta adresa si alt numar de port.\n");
            portServer = 0;
            adresaServer = null;
          }
        }
      }

      // 
      else if ((ev.getSource() == stabilireNumeUser) || 
               (ev.getSource() == intrareNumeUser)) {
        numeUser = intrareNumeUser.getText();
        intrareNumeUser.setEditable(false);
        setTitle(numeUser);
      }

    }      
  };
  
  /**
   * Metoda principala a firului care receptioneaza mesaje de la server
   */
  public void run() {     

    try {

      // Obtinerea adresei locale ca obiect InetAddress
      localHost = InetAddress.getLocalHost();

      // Obtinerea formei String a adresei locale
      adresaLocala = localHost.getHostAddress();

      // Obtinerea numarului de port local
      portLocal = socket.getLocalPort();

      // Bucla infinita - repetare la infinit
      while (true) {                                 

        // Citirea mesajului din fluxul de intrare dinspre server
        String line = inRetea.readUTF ();        

        // Adaugare textului primit in iesirea de text    
        outTextGrafic.append (line + "\n");      
      }
    } 
    // In cazul unei erori legata de conexiune
    catch (IOException ex) {      
      // Afisarea exceptiei
      ex.printStackTrace ();           
    } 
    // Curatenie finala
    finally {                                      
      // Inchiderea firului de receptie curent
      firReceptie = null;                            
      // Ascunderea intrarii de text
      inTextGrafic.setVisible(false);                
      // Reasezarea interfetei grafice
      validate ();                                   
      try {
        // Inchiderea fluxului de iesire spre retea
        outRetea.close ();                           
        } 
      // In cazul unei erori legata de conexiune
      catch (IOException ex) {
        // Afisarea exceptiei
        ex.printStackTrace ();
      }
    }   
  }
  
  /**
   * Metoda principala - creaza socketul, fluxurile si lanseaza clientul
   * @param args[]
   * @param args
   * @throws java.io.IOException
   */
  public static void main(java.lang.String[] args) throws IOException {
  
    new ClientDeChat_v1plus (); 

  }
}
