//Source file: C:\\sw_dev\\chat\\TratareClientDeChat_v1.java

/**
 * Server chat simplu - componenta de tratare a unei conexiuni
 */
import java.net.*;
import java.io.*;
import java.util.*;
import java.util.Vector;

public class TratareClientDeChat_v1 extends Thread {
  
  /**
   * Vector de referinte la obiecte care trateaza clienti (ptr. inregistrare)
   */
  private static Vector listaFire = new Vector ();
  
  /**
   * Socket flux (TCP)
   */
  private Socket socket;
  
  /**
   * Flux de intrare dinspre retea
   */
  private DataInputStream inRetea;
  
  /**
   * Flux de iesire catre retea
   */
  private DataOutputStream outRetea;
  
  /**
   * Initializeaza obiectul (firul) care trateaza un nou client
   * 
   * @param socket
   * @throws java.io.IOException
   */
  public TratareClientDeChat_v1(Socket socket) throws IOException {
    this.socket = socket;
    inRetea = new DataInputStream (new BufferedInputStream (socket.getInputStream ()));
    outRetea = new DataOutputStream (new BufferedOutputStream (socket.getOutputStream ()));   
  }
  
  /**
   * Metoda principala a firului de executie.
   * Primeste mesajele si apeleaza difuzarea lor.
   */
  public void run() {                      

    try {
      // Inregistrarea firului curent in lista (Vector)
      listaFire.addElement (this);           

      System.out.print("\nNou fir de executie ...");
      System.out.println(this.toString());
      System.out.println(listaFire.toString()+"\n");

      // Bucla infinita - repetare la infinit
      while (true) {   
                     
        // Citirea mesajului din fluxul de intrare de la client
        String mesaj = inRetea.readUTF ();

        // Difuzarea catre toti clientii curent inregistrati
        difuzare (mesaj);                    
      }
    } 
    // In cazul unei erori legata de conexiune
    catch (IOException ex) {      
      // Afisare exceptie
      ex.printStackTrace ();           
    } 
    // Curatenie finala
    finally {                          
      // Eliminarea firului curent din lista (Vector)
      listaFire.removeElement (this);  

      System.out.print("\nFir de executie eliminat...");
      System.out.println(this.toString());
      System.out.println(listaFire.toString()+"\n");

      try {
        // Inchiderea socketului
        socket.close ();               
      } 
      // In cazul unei erori legata de conexiune
      catch (IOException ex) {      
        // Afisarea exceptiei
        ex.printStackTrace ();           
      } 
    }   
  }
  
  /**
   * Difuzeaza mesajul primit catre clienti
   * 
   * @param mesaj
   */
  private static void difuzare(String mesaj) {   

    // Enumerare creata pornind de la lista firelor de executie    
    Enumeration enum = listaFire.elements ();           

    // Cat timp mai sunt elemente in enumerare
    while (enum.hasMoreElements ()) {            

      // Referinta catre firul curent initializata cu null
      TratareClientDeChat_v1 firDestinatie = null;                     

      // Protectie la acces concurent la Vectorul firelor
      synchronized (listaFire) {               
   
        // Obtinerea referintei catre firul curent 
        firDestinatie = (TratareClientDeChat_v1) enum.nextElement ();  
      }                                          

      // Daca referinta e valida
      if (firDestinatie  != null) {                          

        try {
         // Protectie la acces concurent la fluxul de iesire
         synchronized (firDestinatie.outRetea) {            

            // Scrierea mesajului in fluxul de iesire al firului curent
            firDestinatie.outRetea.writeUTF (mesaj);      
          }

          // Fortarea trimiterii mesajului
          firDestinatie.outRetea.flush ();                  
        } 

        // In cazul unei erori legata de conexiune
        catch (IOException ex) {
          // Inchiderea firului curent
          firDestinatie.stop ();                            
        }

      }
    }   
  }
}
